#!/usr/bin/env python3
import os,sys
paths = os.environ.get('PATH')
paths = paths.split(':')
for path in paths:
	if 'scripts' in path:
		sys.path.append(path)
		break
from Utils import *
import numpy as np
import argparse
from pyquaternion import Quaternion
import matplotlib.pyplot as plt
from matplotlib.patches import Circle
from matplotlib.collections import PatchCollection
import matplotlib
matplotlib.rc('text.latex',preamble=r'\usepackage{amsmath}\usepackage{bm}')
matplotlib.rc('text', usetex=True)

def GetTimesAnglesForces(where='.',save_every=25):
	data = np.loadtxt(where + '/force.dat',skiprows=1)
	iterations = data[:,0]
	times = np.array(iterations/save_every)
	forces = data[:,[6,7,8]]
	forces = np.linalg.norm(forces,axis=1)
	force_ref = GetReferenceForce(where=where)
	forces = forces/force_ref
	qs = [Quaternion(qw,qx,qy,qz).normalised for (qx,qy,qz,qw) in data[:,[2,3,4,5]]]
	angles = np.array([q.degrees for q in qs])
	#print("time ranges from t = {} to {}".format(times[0],times[-1]))
	return times[:-1],angles[:-1],forces[:-1]

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('--sides',type=StrToNumpy,default="3:8",help='number of sides')
parser.add_argument('--theta',type=int,default=70,help='contact angle')
parser.add_argument('--ec',type=float,default=4,help='Ec')
parser.add_argument('--every',type=int,default=25,help='every')
args = parser.parse_args()

#~~~~~~
#  Ax
#~~~~~~
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.12,0.12,0.5,0.4])
fontSize = 10
frameWidth = 0.5
xticks = np.arange(0,101,20)
xticklabels = [r'${}^\circ$'.format(x) for x in xticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_xticks(xticks)
ax.set_xticklabels(xticklabels)
yticks =  np.linspace(0,6,num=6)
yticklabels = [r'${:.0f}$'.format(y) for y in yticks]
ax.set_ylim(yticks[0],yticks[-1])
ax.set_yticks(yticks)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$\alpha$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$|\bm{F}_e|/F_s$',fontsize=10,labelpad=4)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Plot force against angle
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sides = args.sides
colors = ['#c6dbef','#9ecae1','#6baed6','#3182bd','#08519c']
for i,side in enumerate(sides[::-1]):
	times,angles,forces = GetTimesAnglesForces(where='side'+str(side))
	ax.plot(angles,
	        forces,
	        "-",
	        color = colors[i],
	        label = r'$N = {}$'.format(side))
	print('  side = {}, alpha(t->infty) = {:.2f}'.format(side,angles[-1]))
handles, labels = ax.get_legend_handles_labels()
ax.legend(handles[::-1],
          labels[::-1],
          ncol=2,
          bbox_to_anchor=(0.01,0.98),
          loc='upper left',
          # ~ frameon = False,
          framealpha=1,
          handletextpad = 0.2,
          handlelength = 0.5).set_zorder(0)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  define which data to show
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
my_path = 'side'+str(sides[0])

#~~~~~~~~~~~~~~~~
#  Double check
#~~~~~~~~~~~~~~~~
# ~ for i,side in enumerate(sides):
	# ~ times = np.arange(0,800,2)
	# ~ forces = np.copy(times).astype(float)
	# ~ angles = np.copy(times).astype(float)
	# ~ for i,timei in enumerate(times):
		# ~ F,_ = GetRealCapillaryForce(where='side'+str(side),theta=args.theta,ec=args.ec,time=timei,info=False)
		# ~ Fmag = np.linalg.norm(F)
		# ~ F0 = GetReferenceForce(where='side'+str(side),time=timei)
		# ~ forces[i] = Fmag/F0
		# ~ _,_,_,_,_,_,q = GetSide(where='side'+str(side),time=timei)
		# ~ angles[i] = q.degrees
	# ~ ax.plot(angles,forces,'o',markersize=2,color='C0')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Read side triangle without transform
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
A,B,C,_,_,_,_ = GetSide(where=my_path,time=0,transform=False)
xs_side = [A[1],B[1],C[1]]
ys_side = [A[0],B[0],C[0]]
xs_side = np.hstack((xs_side,xs_side[0]))
ys_side = np.hstack((ys_side,ys_side[0]))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Create new axes + plot triangle side
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
axs = []
x0,x1 = ax.get_xlim();
y0,y1 = ax.get_ylim();
x0,y0 = TransformAxToFig(ax,x0,y0)
x1,y1 = TransformAxToFig(ax,x1,y1)
alpha = (xs_side.max()-xs_side.min())/(ys_side.max()-ys_side.min()) #aspect ratio
ly = 0.45*(y1-y0)
lx = alpha*ly
dx = lx/2
x0_all = [x1+1*dx,x1+2*dx+1*lx,x1+3*dx+2*lx,\
          x1+1*dx,x1+2*dx+1*lx,x1+3*dx+2*lx]
y0_all = [y1-ly,y1-ly,y1-ly,y0,y0,y0]
for i,(x0,y0) in enumerate(zip(x0_all,y0_all)):
	axs.append(fig.add_axes([x0,y0,lx,ly]))
	axs[-1].axis('off')
	axs[-1].set_xlim([xs_side.min(),xs_side.max()])
	axs[-1].set_ylim([ys_side.min(),ys_side.max()])
	axs[-1].set_xticks([])
	axs[-1].set_yticks([])
	axs[-1].plot(xs_side,ys_side,color='black',linewidth=0.5)

#~~~~~~~~~~~~~~~~~
#  Select points
#~~~~~~~~~~~~~~~~~
times,angles,forces = GetTimesAnglesForces(where=my_path)
angles_estimated = np.array([5,38,60,84,88,98],dtype=int)
indices_estimated = np.array([np.argmin(np.abs(angles-anglei)) for anglei in angles_estimated],dtype=int)
print("estimated times to pick (float):",np.take(times,indices_estimated))
print("estimated times to pick (int):",np.take(times,indices_estimated).astype(int))
times_selected = np.take(times,indices_estimated).astype(int)
indices = np.ravel([np.argwhere(times==timei) for timei in times_selected])
indices[-1] = -1
times_selected = np.take(times,indices).astype(int)
angles_selected = np.take(angles,indices)
forces_selected = np.take(forces,indices)
# ~ print("indices_selected =",indices)
# ~ with np.printoptions(precision=2,suppress=True):
	# ~ print("times_selected =",times_selected)
	# ~ print("angles_selected =",angles_selected)
	# ~ print("forces_selected =",forces_selected)
ax.plot(angles_selected[:-1],
        forces_selected[:-1],
        marker = 'o',
        markersize = 5,
        markerfacecolor = 'None',
        markeredgecolor = 'C1',
        markeredgewidth = 1,
        linewidth = 0)
labels = ['a','b','c','d','e','f']
for label,x,y in zip (labels[:-1],angles_selected[:-1],forces_selected[:-1]):
	ax.text(x-2,y+0.1,r'${}$'.format(label),ha='center',va='bottom',color='C1')

#~~~~~~~~~~~~~~~~~~
#  Plot polylines
#~~~~~~~~~~~~~~~~~~
for i,(axi,timei,anglei) in enumerate(zip(axs,times_selected,angles_selected)):
	_,_,_,h,n,c,q = GetSide(where=my_path,time=timei,transform=True)
	R = (q.inverse).rotation_matrix
	polylines = ImportPolylines(where=my_path,name='tpcl',time=timei,cloudsize=30,info=False)
	for polyline in polylines:
		if polyline.size>0:
			polyline = np.array([R.dot(pt-c) for pt in polyline])
			xs = polyline[:,1]
			ys = polyline[:,0]
			axi.plot(xs,ys,color='C1')
	force, torque = GetRealCapillaryForce(where = my_path,
	                                      name = 'tpcl',
	                                      theta = args.theta,
	                                      ec = args.ec,
	                                      time = timei,
	                                      info = False)
	force = np.linalg.norm(force)
	force_ref = GetReferenceForce(where=my_path)
	print("Force (c++/python) = {:.3f}\t{:.3f}".format(force/force_ref,forces_selected[i]))
	bb = axi.get_position()
	x0,y0 = bb.x0,bb.y0
	x1,y1 = bb.x1,bb.y1
	alpha = q.degrees
	ax.text((x0+x1)/2,0.95*y0,r'$\alpha = {:.0f}^\circ$'.format(alpha),ha='center',va='top',transform=fig.transFigure)
	ax.text((x0+x1)/2,
	        y0+0.5*(y1-y0),
	        r'${}$'.format(labels[i]),
	        ha='center',
	        va='center',
	        color='C1',
	        transform=fig.transFigure)
A=[95,5]
B=[95,5.5]
Arrow(ax,A,B,color='C1',arrowstyle='-|>',head_width=0.2,head_length=0.4)
ax.text(B[0],B[1],r'$f$',ha='right',va='bottom',color='C1')

fig.savefig('fig.pdf')
plt.close(fig)
