#!/usr/bin/env python3
import os,sys
paths = os.environ.get('PATH')
paths = paths.split(':')
for path in paths:
	if 'scripts' in path:
		sys.path.append(path)
		break
from Utils import *
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import argparse
from pyquaternion import Quaternion
from scipy.optimize import curve_fit
matplotlib.rc('text', usetex=True)
matplotlib.rc('text.latex', preamble=r'\usepackage{amsmath}')

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('--theta',type=int,default=90,help='contact angle')
parser.add_argument('--lastTime',type=int,default=790,help='time')
args = parser.parse_args()

#~~~~~~
#  Ax
#~~~~~~
fig, ax = plt.subplots()
my_font_size = 14
my_frame_width = 0.5
xticks = np.linspace(0,3.5,8)
xticklabels = [r'${:.1f}$'.format(x) for x in xticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_xticks(xticks)
ax.set_xticklabels(xticklabels)
yticks =  np.arange(0,111,25)
yticklabels = [r'${:d}^\circ$'.format(int(y)) for y in yticks]
ax.set_ylim(yticks[0],yticks[-1])
ax.set_yticks(yticks)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$Ec$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$\alpha_\infty$',fontsize=10,labelpad=2)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=my_frame_width,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=my_frame_width,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(my_frame_width)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(my_font_size)

#~~~~~~~~~~~~~
#  Functions
#~~~~~~~~~~~~~
def ImportData(ecs=np.arange(0,400,10)/100,theta=90,time=399):
	#alpha = alphas[side][ec]
	def GetAlpha(my_path,time):
		side_name = GetFileName(my_path + "/side0",time)
		if not os.path.isfile(side_name):
			print("  {} does not exist".format(side_name))
			return np.nan
		else:
			data = np.loadtxt(side_name)
			q = Quaternion(data[19],data[16],data[17],data[18])
			return  q.degrees
	folders_s3 = ["ca{}/side3/ec{:.1f}".format(theta,ec) for ec in ecs] #.1f / .2f
	folders_s4 = ["ca{}/side4/ec{:.1f}".format(theta,ec) for ec in ecs]
	folders_s5 = ["ca{}/side5/ec{:.1f}".format(theta,ec) for ec in ecs]
	folders_s6 = ["ca{}/side6/ec{:.1f}".format(theta,ec) for ec in ecs]
	folders_s7 = ["ca{}/side7/ec{:.1f}".format(theta,ec) for ec in ecs]
	folders = [folders_s3,folders_s4,folders_s5,folders_s6,folders_s7]
	alphas = np.empty((len(folders),len(folders[0])))
	for (i,j),alpha in np.ndenumerate(alphas):
		alphas[i][j] = GetAlpha(folders[i][j], time)
	return np.vstack(([ecs]*len(folders))),alphas

def CorrectData(ecs,alphas):
	zeros = np.zeros((alphas.shape[0],1))
	ecs = np.hstack((zeros,ecs))
	alphas = np.hstack((zeros,alphas))
	ecs    = [erow[~np.isnan(arow)] for erow,arow in zip(ecs,alphas)]
	alphas = [arow[~np.isnan(arow)] for      arow in         alphas ]
	return ecs,alphas

#~~~~~~~~
#  Data
#~~~~~~~~
ecs,alphas = ImportData(theta=args.theta,time=args.lastTime)
#ecs,alphas = CorrectData(ecs,alphas)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Select for curve-fitting data 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ecs_fit = []
alphas_fit = []
for rowe,rowa in zip(ecs,alphas):
	#~~~~~~~~
	#Method 1
	#~~~~~~~~
	#diffa = np.diff(rowa)
	#crop = np.argwhere(diffa<2)
	#if crop.size == 0:
		#ecs_fit.append(rowe)
		#alphas_fit.append(rowa)
	#else:
		#crop = crop.flatten()[0]
		#ecs_fit.append(rowe[:crop+1])
		#alphas_fit.append(rowa[:crop+1])
	#~~~~~~~~
	#Method 2
	#~~~~~~~~
	alpha_max = np.max(rowa)
	crop = np.argwhere(rowa<0.90*alpha_max)
	crop = crop.flatten()[-1]
	ecs_fit.append(rowe[:crop+1])
	alphas_fit.append(rowa[:crop+1])

#~~~~~~~~
#  Plot
#~~~~~~~~
colors = ['#c6dbef','#9ecae1','#6baed6','#3182bd','#08519c']
sides = [3,4,5,6,7]
for i,(erow,arow) in enumerate(zip(ecs    ,alphas    )):
#for i,(erow,arow) in enumerate(zip(ecs_fit,alphas_fit)):
	ax.plot(erow,
	        arow,
	        marker = 'o',
	        markersize = 5,
	        markerfacecolor = 'white',
	        markeredgecolor = colors[i],
	        markeredgewidth = 1,
	        linewidth = 0,
	        label = r'$N = {}$'.format(sides[i]))

#~~~~~~~
#  Fit
#~~~~~~~
def func(x,a,b):
	return a*(x)+b*(x**2)
ecs_tilde = np.empty((len(ecs)))
for i,(erow,arow) in enumerate(zip(ecs_fit,alphas_fit)):
	if rowa.size:
		y_infty = alphas[0][-1]
		popt, pcov = curve_fit(func,erow,arow)
		xs = np.linspace(0,4,num=1000)
		ys = func(xs,*popt)
		ecs_tilde[i] = (xs[ys > y_infty])[0]
		ys[ys > y_infty] = y_infty
		ax.plot(xs,ys,'-',color=colors[i],zorder=0)
print('  tilde(Ec) = ',ecs_tilde)

#~~~~~~~~~~~~~~
#  Annotation
#~~~~~~~~~~~~~~
for i,ec_tilde in enumerate(ecs_tilde):
	alpha_infty = alphas[0][-1]
	A = [ec_tilde+0.1,alpha_infty-10]
	B = [ec_tilde    ,alpha_infty+0 ]
	Arrow(ax,A,B,arrowstyle='-|>',color=colors[i],head_width=0.2,head_length=0.4,shrinkA=4.,shrinkB=6.,text=r'$\tilde{E}c$',zorder=10)

#~~~~~~~~~~
#  Legend
#~~~~~~~~~~
ax.legend()
handles, labels = ax.get_legend_handles_labels()
#sort=[3,2,1,0]
#labels = [labels[i] for i in sort]
#handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      fontsize = my_font_size,
	      loc = 'lower right',
	      frameon = False,
	      handletextpad = 0.2,
	      handlelength = 2.5)

fig.savefig('fig.pdf')
plt.close(fig)
