#!/usr/bin/env python3
import numpy as np
from scipy.optimize import fsolve
import matplotlib.pyplot as plt
import subprocess
import re
from scipy.optimize import minimize
from mpl_toolkits.mplot3d import Axes3D
import csv
from scipy import interpolate
from intersect import intersection
import argparse

# Sample drop volume V(z,r) and contact angle theta(z,r)
# z is altitude of drop centre
# r drop radius
# With the space sampling, it is possible to find the drop properties (z,r)
# make sure first the code src/Sphere.cpp has been compiled

def StrToBool(s):
	if isinstance(s,bool):
		return s
	if s.lower() in ('yes', 'true', 't', 'y', '1'):
		return True
	elif s.lower() in ('no', 'false', 'f', 'n', '0'):
		return False
	else:
		raise argparse.ArgumentTypeError('Boolean value expected.')

def StrToNumpy(s):
	if not s:
		return np.array([])
	else:
		l = list(map(int,s.split(':')))
		return np.array(l) if len(l)==1 else  np.arange(l[0],l[1]) if len(l)==2 else np.arange(l[0],l[1],l[2])

def to_rad(angle):
	return angle*np.pi/180

def to_deg(angle):
	return angle*180/np.pi

def flatten_nested_list(mat):
	return [element for row in mat for element in row]

def write_nested_list(filename,nested_list):
	with open(filename+'.csv', "w") as f:
		wr = csv.writer(f)
		wr.writerows(nested_list)

def read_nested_list(filename):
	nested_list = []
	with open(filename+'.csv', "r") as f:
		reader = csv.reader(f)
		for row in reader:
			nested_list.append(np.array(row,dtype=float))
	return nested_list

def update_template(template=None,centre=[-1,-1,-1],apothem=-1,thickness=-1,side=-1,alpha=-1):
	#Template object in the form of a dictionary
	tmp = {}
	if template is None:
		tmp = {
			'centre': np.array([75,75,6],dtype='f'),
			'apothem': 7.5,
			'thickness': 2,
			'side': 50.559,
			'alpha': to_rad(30)
		}
	if not np.array_equal(centre,[-1,-1,-1]) : tmp['centre'] = np.array(centre)
	if apothem  !=-1 : tmp['apothem']   = apothem
	if thickness!=-1 : tmp['thickness'] = thickness
	if side     !=-1 : tmp['side']      = side
	if alpha    !=-1 : tmp['alpha']     = alpha
	#xa,ya use for python plot and formulae
	tmp['xc'] = tmp['centre'][0]
	tmp['yc'] = tmp['centre'][2] + tmp['thickness']/2
	tmp['xa'] = tmp['xc'] + tmp['apothem']
	tmp['ya'] = tmp['yc']
	return tmp

def update_drop(drop=None,centre=[-1,-1,-1],x=-1,y=-1,z=-1,r=-1):
	#Drop object in the form of a dictionary
	tmp = {}
	if drop is None:
		tmp = {
			'centre': np.array([75,75,0],dtype='f'),
			'radius': 18
		}
	if not np.array_equal(centre,[-1,-1,-1]) : tmp['centre'] = np.array(centre)
	if x !=-1 : tmp['centre'][0] = x
	if y !=-1 : tmp['centre'][1] = y
	if z !=-1 : tmp['centre'][2] = z
	if r !=-1 : tmp['radius']    = r
	#xd,yd,rd use for python plot and formulae
	tmp['xd'] = tmp['centre'][0]
	tmp['yd'] = tmp['centre'][2]
	tmp['rd'] = tmp['radius']
	return tmp

def eq_side(x,y,xa,ya,alpha):
	# inclined side equation
	# x,y    point (x,y) = (xa,ya) + t.(ux,uy)
	# xa,ya  apothem point
	# alpha  folding angle
	# https://emedia.rmit.edu.au/learninglab/content/v6-vector-equation-line
	ux = np.cos(alpha)
	uy = np.sin(alpha)
	#if np.linalg.norm(ux)<1e-6 or np.linalg.norm(uy)<1e-6 or np.linalg.norm([ux,uy]):
		#print("error")
	return (x-xa)/ux-(y-ya)/uy # = 0

def eq_drop(x,y,xd,yd,rd):
	# drop equation
	# x,y   point (x,y) on drop arc
	# rd    radius
	# xd,yd drop centre
	return (x-xd)**2+(y-yd)**2-rd**2 # = 0

def solution_exists(template,drop):
	# Check if drop intersects inclined side of the template
	xa = template['xa']
	ya = template['ya']
	xd = drop['xd']
	yd = drop['yd']
	rd = drop['rd']
	r2 = (xa-xd)**2 + (ya-yd)**2  #apothem is in full drop
	if (r2<rd**2):
		return True
	else:
		print("  Error: drop does not intersect the template side")
		print("    template =",template)
		print("    drop =",drop)
		return False

def find_contact_angle(template,drop,return_root=False,info=False):
	#Determine contact angle analytically
	if solution_exists(template=template,drop=drop):
		xa,ya,alpha = template['xa'],template['ya'],template['alpha']
		xd,yd,rd = drop['xd'],drop['yd'],drop['rd']
		def func(X):
			# X = [x,y] intersection between drop with side lines
			# There can be 2 solutions to the equation set
			return [eq_side(X[0],X[1],xa,ya,alpha),eq_drop(X[0],X[1],xd,yd,rd)]
		x_approx = xa + rd*np.cos(alpha)
		y_approx = ya + rd*np.sin(alpha)
		x_root,y_root = fsolve(func,[x_approx,y_approx])
		if y_root<ya:
			print('  Error: wrong solution taken for drop radius = {:.2f} and drop altitude =  {:.2f}'.format(rd,yd))
		if info:
			print('  x_root =',x_root)
			print('  y_root =',y_root)
			print('  f(root) =',func([x_root,y_root]))
		vec_temp = np.array([xa-x_root,ya-y_root])
		vec_drop = np.array([-(y_root-yd),x_root-xd]) #t=(x,y) -> n=(-y,x)
		vec_temp = vec_temp/np.linalg.norm(vec_temp)
		vec_drop = vec_drop/np.linalg.norm(vec_drop)
		dotp = np.dot(vec_drop,vec_temp)
		if return_root:
			return np.arccos(dotp),x_root,y_root
		return np.arccos(dotp)
	else:
		return -1

def sphere_cpp(template,drop):
	cmd = ['Sphere',
		' --templatec {:.4f},{:.4f},{:.4f}'.format(*template['centre']),\
		' --apothem {:.4f}'.format(template['apothem']),\
		' --side {:.4f}'.format(template['side']),\
		' --thickness {:.4f}'.format(template['thickness']),\
		' --alpha {:.4f}'.format(template['alpha']),\
		' --dropc {:.4f},{:.4f},{:.4f}'.format(*drop['centre']),\
		' --dropr {:.4f}'.format(drop['radius'])]
	return ''.join(cmd)

def find_volume(template,drop,info=False):
	#Determine drop volume using Sphere.cpp
	def volume(p):
		vol = re.search("volume =.*",p.stdout).group()
		vol = re.split("\s",vol)
		return float(vol[-1])
	if solution_exists(template=template,drop=drop):
		cmd = sphere_cpp(template,drop)
		p = subprocess.run(cmd,capture_output=True,text=True,shell=True)
		if info:
			print(cmd)
			print(p.stdout)
		vol = volume(p)
		if (vol<1e-6):
			#3D tesselation probably crashed
			print('  ' + cmd)
			print('  Warning: I could could not be determine the volume. I perform a second attempt')
			small = 0.0001;
			drop['centre'][2] = drop['centre'][2] * (1+small)
			cmd = sphere_cpp(template,drop)
			p = subprocess.run(cmd,capture_output=True,text=True,shell=True)
			vol = volume(p)
			if (vol<1e-6):
				print('  ' + cmd)
				print("  Error: volume determination failed")
		return vol
	else:
		return -1

def find_tpcl_length(template,drop,info=False):
	#Determine length of the tree-phase contact line (tpcl) using Sphere.cpp
	def tpcl_length(p):
		length = re.search("length =.*", p.stdout).group()
		length = re.split("\s",length)
		return float(length[-1])
	if solution_exists(template=template,drop=drop):
		cmd = sphere_cpp(template,drop)
		p = subprocess.run(cmd,capture_output=True,text=True,shell=True)
		if info:
			print(cmd)
			print(p.stdout)
		length = tpcl_length(p)
		if (length<1e-6):
			#3D tesselation probably crashed
			print('  Warning: I could could not be determine the length of tpcl. I perform a second attempt')
			print('  ' + cmd)
			small = 0.0001;
			drop['centre'][2] = drop['centre'][2] * (1+small)
			cmd = sphere_cpp(template,drop)
			p = subprocess.run(cmd,capture_output=True,text=True,shell=True)
			length = tpcl_length(p)
			if (length<1e-6):
				print("  Error: tpcl length determination failed")
		return length
	else:
		return -1

def run_doe(template,dr=0.5,info=False):
	# doe = design of experiments
	# sample functions theta(z,r) and volume(z,r)
	# z = drop altitude
	# r = drop radius
	# dr = sampling length
	def find_zmin_zmax(template,drop_radius,info=False):
		# Given the drop radius, find the vertical min/max drop centre altitude,
		# for which an analytical solution exists
		apothem = template['apothem']
		if drop_radius < apothem:
			print("  Error: drop_radius = {:.2f} < apothem = {:.2f}".format(radius,apothem))
			return -1,-1
		z0 = template['centre'][2] + template['thickness']/2
		eps = 1e-4
		h = np.sqrt(drop_radius**2-apothem**2)
		z_min,z_max = z0-h,z0+h
		z_min,z_max = z_min+eps,z_max-eps
		return z_min,z_max
	def prepare_doe(template,dr,info=False):
		# sample drop radii and drop altitudes
		# rss list of drop radii
		# yss list of drop positions given the radii rds
		r_min = template['apothem']
		r_max = 5.*r_min
		num = round((r_max-r_min)/dr)+1
		rs = np.linspace(r_min,r_max,num,endpoint=True)
		zss = []
		for r in rs:
			z_min,z_max = find_zmin_zmax(template,drop_radius=r,info=info)
			num = round((z_max-z_min)/dr)+1
			zs = np.linspace(z_min,z_max,num,endpoint=True)
			zss.append(zs)
		if info:
			r_min = rs[ 0]
			r_max = rs[-1]
			dr = rs[1]-rs[0]
			print('  r_min = {:.2f} r_max = {:.2f} dr = {:.3f} size = {}'.format(r_min,r_max,dr,rs.size))
			for r,zs in zip(rs,zss):
				if zs.size==1:
					print('  r = {:.2f} z_min = z_max = {:.2f} size = 1 (case removed)'.format(r,zs[0]))
				else:
					z_min = zs[ 0]
					z_max = zs[-1]
					dr = zs[1]-zs[0]
					print('  r = {:.2f} z_min = {:.2f} z_max = {:.2f} dr = {:.3f} size = {}'.format(r,z_min,z_max,dr,zs.size))
		rs  = [r  for zs,r in zip(zss,rs) if zs.size>1] #remove special case
		zss = [zs for zs   in     zss     if zs.size>1] #remove special case
		rss = [r*np.ones(zs.shape) for r,zs in zip(rs,zss)]
		return rss,zss
	rss,zss = prepare_doe(template,dr,info=info)
	thetass = [np.empty(zs.shape) for zs in zss]
	volumess = [np.empty(zs.shape) for zs in zss]
	for i,(rs,zs) in enumerate(zip(rss,zss)):
		print('  drop radius = {:.2f} ({}/{})'.format(rs[0],i,len(rss)-1))
		for j,(r,z) in enumerate(zip(rs,zs)):
			drop = update_drop(r=r,z=z)
			theta = find_contact_angle(template,drop)
			volume = find_volume(template,drop)
			if volume<1e-6:
				cmd = find_volume(template,drop,print_command_only=True)
				print('  Warning: volume too small with r = {} and z = {}'.format(r,z))
				print('  Warning: {}'.format(cmd))
			thetass[i][j] = to_deg(theta)
			volumess[i][j] = volume
	write_nested_list("rss",rss)
	write_nested_list("zss",zss)
	write_nested_list("thetass",thetass)
	write_nested_list("volumess",volumess)
	return rss,zss,thetass,volumess

def load_doe():
	# load data from design of experiments
	rss = read_nested_list("rss")
	zss = read_nested_list("zss")
	thetass = read_nested_list("thetass")
	volumess = read_nested_list("volumess")
	return rss,zss,thetass,volumess

def slices_doe(rss,zss,thetass,volumess,theta=70,volume=2000):
	# curve z(r) obtained by slicing the 3d plots theta(r,z) and volume(r,z)
	# with theta=cst and volume=cst
	def slide2D(rss,zss,fss,f):
		#fss = thetass or volumess
		rs_interp = []
		zs_interp = []
		for rs,zs,fs in zip(rss,zss,fss):
				if fs[0]<f<fs[-1]:
					interp = interpolate.interp1d(fs,zs,kind='cubic')
					r_interp = rs[0]
					z_interp = interp(f).item()
					rs_interp.append(rs[0])
					zs_interp.append(z_interp)
		rs_interp = np.array(rs_interp)
		zs_interp = np.array(zs_interp)
		# slice=[r0,r1,r2,...][z0,z1,z2,...][theta,theta,theta,...]
		return [rs_interp,zs_interp,np.full(rs_interp.shape,f)]
	slice_theta = slide2D(rss,zss,thetass,theta)
	slice_volume = slide2D(rss,zss,volumess,volume)
	slices = [slice_theta,slice_volume]
	return slices

def plot3D_doe_results(rss,zss,thetass,volumess,slices=None):
	#plot 3d plots successively, that are 1) theta(z,r) and then 2) volume(z,r)
	fig = plt.figure(0)
	ax = fig.add_subplot(projection='3d')
	print('  plot 1')
	xs = flatten_nested_list(rss)
	ys = flatten_nested_list(zss)
	zs = flatten_nested_list(thetass)
	ax.scatter(xs,ys,zs,color='C0')
	if slices is not None:
		xs_interp = slices[0][0]
		ys_interp = slices[0][1]
		zs_interp = slices[0][2]
		ax.plot(xs_interp,ys_interp,zs_interp,'-',color='C1')
	ax.set_xlabel('drop radius')
	ax.set_ylabel('drop position')
	ax.set_zlabel('contact angle')
	ax.set_box_aspect((1, 1, 1))
	plt.show()
	print('  plot 2')
	fig = plt.figure(1)
	ax = fig.add_subplot(projection='3d')
	xs = flatten_nested_list(rss)
	ys = flatten_nested_list(zss)
	zs = flatten_nested_list(volumess)
	ax.scatter(xs,ys,zs)
	if slices is not None:
		xs_interp = slices[1][0]
		ys_interp = slices[1][1]
		zs_interp = slices[1][2]
		ax.plot(xs_interp,ys_interp,zs_interp,'-',color='C1')
	ax.set_xlabel('drop radius')
	ax.set_ylabel('drop position')
	ax.set_zlabel('volume')
	ax.set_box_aspect((1, 1, 1))
	plt.show()

def plot2D_drop(ax,template,drop,info=False):
	#Plot drop and template in 2D
	#This is used for determination of the contact angle
	xa,ya,alpha,a,s = template['xa'],template['ya'],template['alpha'],template['apothem'],template['side']
	xs_template = [xa-2*a-s*np.cos(alpha),xa-2*a,xa,xa+s*np.cos(alpha)]
	ys_template = [ya+    s*np.sin(alpha),ya    ,ya,ya+s*np.sin(alpha)]
	t = np.linspace(0,2*np.pi,1000)
	xd,yd,rd = drop['xd'],drop['yd'],drop['rd']
	xs_drop = xd + rd * np.cos(t)
	ys_drop = yd + rd * np.sin(t)
	if solution_exists(template,drop):
		_,x_root,y_root = find_contact_angle(template,drop,return_root=True,info=info)
		xc = template['xc']
		ax.plot(x_root,y_root,'s',color='C1')
		ax.plot(xc-(x_root-xc),y_root,'s',color='C1')
		xs_drop_up   = xs_drop[ys_drop>y_root];
		ys_drop_up   = ys_drop[ys_drop>y_root];
		xs_drop_down = xs_drop[ys_drop<y_root];
		ys_drop_down = ys_drop[ys_drop<y_root];
		ax.plot(xs_drop_up,ys_drop_up,'o',color=color,markersize=0.1)
		ax.plot(xs_drop_down,ys_drop_down,'o',color=color,markersize=0.01)
	else:
		print("plot full drop because no solution exists")
		ax.plot(xs_drop,ys_drop,'o',color=color,markersize=0.1)
	#ax.set_xlim(xs_template[0],xs_template[-1])
	#ax.set_ylim(ys_template[0],ys_template[-1])
	ax.set_aspect('equal')
	ax.plot(xs_template,ys_template,'-')
	ax.plot([xd],[yd],'+',color='b')

#Create template
template = update_template()

#Design of experiments
parser = argparse.ArgumentParser()
parser.add_argument('--new',type=StrToBool,default='n',help='new')
parser.add_argument('--dr',type=float,default=1.5,help='dr')
parser.add_argument('--volumes',type=StrToNumpy,default='1000',help='drop volumes')
parser.add_argument('--theta',type=float,default=70,help='contact angle')
parser.add_argument('--plot',type=StrToBool,default='no',help='3D plot')
parser.add_argument('--silent',type=StrToBool,default='no',help='silence  output')
args = parser.parse_args()
if args.new:
	print("  new design of experiments")
	rss,zss,thetass,volumess = run_doe(template,dr=args.dr,info=True)
	if args.plot:
		plot3D_doe_results(rss,zss,thetass,volumess)
else:
	rss,zss,thetass,volumess = load_doe()
	if args.plot:
		slices = slices_doe(rss,zss,thetass,volumess,theta=args.theta,volume=args.volumes[0])
		plot3D_doe_results(rss,zss,thetass,volumess,slices=slices)

for volume in args.volumes:
	#slices = [slice0,slice1]
	#slice0 = z(r) for theta = cst
	#slice1 = z(r) for volume = cst
	slices = slices_doe(rss,zss,thetass,volumess,theta=args.theta,volume=volume)

	#Interpolate slice 1
	rs0,zs0,thetas = slices[0]
	interp0 = interpolate.interp1d(rs0,zs0,kind='cubic')
	rs0_interp = np.linspace(rs0[0],rs0[-1],num=100,endpoint=True)
	zs0_interp = interp0(rs0_interp)

	#Interpolate slice 2
	rs1,zs1,volumes = slices[1]
	interp1 = interpolate.interp1d(rs1,zs1,kind='cubic')
	rs1_interp = np.linspace(rs1[0],rs1[-1],num=100,endpoint=True)
	zs1_interp = interp1(rs1_interp)

	#Intersect two curves
	rsi,zsi = intersection(rs0_interp, zs0_interp, rs1_interp, zs1_interp)
	if (len(rsi)>0):
		#print('drop radius and drop altitude at intersection =',rsi,zsi)
		#ax.plot(rsi,zsi,'o')
		drop = update_drop(r=rsi.item(),z=zsi.item())
		theta = find_contact_angle(template,drop)
		volume = find_volume(template,drop)
		length = find_tpcl_length(template,drop)
		if not args.silent:
			print("  theta = {:.2f} deg, vol = {:.2f}, L = {:.2f}".format(to_deg(theta),volume,length))
	else:
		print('  Warning: drop radius and drop altitude do not intersect')
		fig = plt.figure(2)
		ax = fig.add_subplot()
		ax.set_xlabel('drop radius')
		ax.set_ylabel('drop position')
		ax.plot(rs0,zs0,'+',color='C0',label='theta={}'.format(thetas[0]))
		ax.plot(rs0_interp,zs0_interp,'-',color='C0',lw=0.5)
		ax.plot(rs1,zs1,'+',color='C1',label='theta={}'.format(thetas[0]))
		ax.plot(rs1_interp,zs1_interp,'-',color='C1',lw=0.5)
		ax.legend()
		plt.show()
