#!/usr/bin/env bash
source Utils.sh
hpc="${HPC}"
owner="${OWNER}/fig8"
finput="Sphere.in"
np="4"
V0="1001"
Vs=($(StrToNumpy "1000:6001:200"))
theta="70"
theory=false
simulation=false
copydata=false
fig=false
clean=false
print=false

function Main()
{
	function PrintUsage()
	{
		echo "Run petsc simulations for comparison to analytical solutions (Figure 8)"
		echo "--help:   help"
		echo "--theta      contact angle"
		echo "--theory     run analytical simulations"
		echo "--simulation make script to run petsc simulations on hpc"
		echo "--copy       copy petsc simulations to local machine and convert"
		echo "--fig        make figure"
		echo "--clean      clean folder"
		echo "--print      print tpcl length"
	}

	function PrintEquilibrium()
	{
		local log=${1}
		if [ -f ${log} ]
		then
			if grep -q 'Equilibrium reached' ${log}
			then
				local n=$(grep 'Iteration' ${log} | tail -1 | sed 's/[^0-9]//g')
				echo "Equilibrium reached after ${n} iterations"
			else
				echo "No equilibrium reached"
			fi
		else
			echo "${log} not found"
		fi
	}
	local longs='help,theta:,theory,simulation,copy,fig,clean,print'
	options=$(getopt --option '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]; then PrintUsage; exit 1; fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; exit 1;;
			--theta) theta=$2; shift 2;;
			--theory) theory=true; shift;;
			--simulation) simulation=true; shift;;
			--copy) copydata=true; shift;;
			--fig) fig=true; shift;;
			--clean) clean=true; shift;;
			--print) print=true; shift;;
			--) shift; break;;
			 *) "Failed parsing options $1";  PrintUsage; exit 1;;
		esac
	done
	unset options
	if ${theory}
	then
		if [ ! -f theory/rss.csv ]
		then
			python3 Sphere.py --new yes --dr 1.5
			mkdir -p theory
			cp rss.csv zss.csv thetass.csv volumess.csv theory
		fi
		cp theory/rss.csv theory/zss.csv theory/thetass.csv theory/volumess.csv .
		local start=$(( ${Vs[0]} ))
		local end=$(( ${Vs[-1]}+1 ))
		local step=$(( ${Vs[1]}-${Vs[0]} ))
		python3 Sphere.py --theta ${theta} --volumes ${start}:${end}:${step}
	fi
	if ${simulation}
	then
		local script="script.sh"
		echo -e "source Utils.sh" > ${script}
		echo -e "for Vi in ${Vs[@]}" >> ${script}
		echo -e "do" >> ${script}
		echo -e "\tmkdir -p V\${Vi}" >> ${script}
		echo -e "\tcp Sphere.in V\${Vi}" >> ${script}
		echo -e "\tUpdateInputFile --in V\${Vi}/${finput} --key \"contact_angle\" --val ${theta}" >> ${script}
		echo -e "\tUpdateInputFile --in V\${Vi}/${finput} --key \"volume\" --val \${Vi}" >> ${script}
		echo -e "\tHemera --in ${finput} --out V\${Vi}/hemera.sh --jobname V\${Vi} --time 1 --np 1" >> ${script}
		echo -e "\techo \"source Utils.sh; ExportData --verbose\" >> V\${Vi}/hemera.sh " >> $script
		echo -e "\twd=\$(pwd); cd V\${Vi}; sbatch hemera.sh; cd \${wd}" >> ${script}
		echo -e "done" >> ${script}
		if OnHPC
		then
			mkdir -p ${owner}
			cp ${script} Sphere.in ${owner}
			echo "run: cd ${owner}"
			echo "run: bash ${script}"
		else
			ssh ${hpc} "mkdir -p ${owner}"
			scp ${script} Sphere.in ${hpc}:${owner}
			echo "run: ssh ${hpc}"
			echo "run: cd ${owner}"
			echo "run: bash ${script}"
		fi
	fi
	if ${copydata}
	then
		if OnHPC
		then
			echo "  Run the following command(s) in interactive mode"
			for Vi in "${Vs[@]}"
			do
				local from="${owner}/V${Vi}"
				local to="$(pwd)/V${Vi}"
				ln -sfv ${from} ${to}
			done
		else
			for Vi in "${Vs[@]}"
			do
				local from="${owner}/V${Vi}"
				local to="$(pwd)/V${Vi}"
				if [[ ! -d ${to} ]]
				then
					echo "  copy/export to ${to}"
					CopyExportedData --hpc ${hpc} --owner ${from} --to ${to} --latestTime #--verbose
				fi
			done
		fi
	fi
	if ${fig}
	then
		python3 fig.py
		pdfcrop fig.pdf fig_bottom.pdf
		pdflatex fig.tex
		pdfcrop fig.pdf fig.pdf
	fi
	if ${clean}
	then
		rm -f script.sh rss.csv thetass.csv volumess.csv zss.csv drop.stl side0.stl sphere.stl template.stl volume.mesh tpcl[0-9][0-9].txt fig.log fig.aux
	fi
	if ${print}
	then
		for Vi in ${Vs[@]}
		do
			if [ -d ca${theta}/V${Vi} ]
			then
				echo "L(V=${Vi}) = $(grep Length ca${theta}/V${Vi}/logV${Vi} | awk '{print $NF}')"
			else
				echo "ca${theta}/V${Vi} does not exist"
			fi
		done
	fi
}

Main "$@"
